import unittest
from unittest.mock import Mock, call, ANY

from kdp_api.model.json_record import JsonRecord
from pandas._testing import assert_frame_equal

import pandas as pd

from kdp_connector.connectors.read import ReadApi
from kdp_api.models import RecordBatch
from kdp_connector.main import KdpConn


class TestRead(unittest.TestCase):

    def setup(self):
        self.jwt = 'eyJhbGciOiJIUzI1NiIs'
        self.workspace_id = 'abc'
        self.dataset_id = '8c6718fa-348f-4019-a28f-c85037fc4b85'
        self.host = 'https://api.koverse.localhost'
        self.batch_size = 50000
        self.starting_record_id = ''

    def test_read_dataset_to_dictionary_list(self):
        self.setup()
        read_api = ReadApi()
        read_api.read_batch_in_sequence = Mock()
        # Returning different values for each call to read_batch_in_sequence.
        read_api.read_batch_in_sequence.side_effect = [self.getRecordBatch(), self.getRecordBatch(),
                                                       self.getRecordBatchNoMoreRecords()]
        kdp_conn = KdpConn()
        config = kdp_conn.create_configuration(self.jwt)
        dictionary_list = read_api.read_dataset_to_dictionary_list(config=config,
                                                                   dataset_id=self.dataset_id,
                                                                   starting_record_id=self.starting_record_id,
                                                                   batch_size=self.batch_size)
        expected_list = []
        expected_list.extend(self.getDataStoreList(self.getRecords()))
        expected_list.extend(self.getDataStoreList(self.getRecords()))
        expected_list.extend(self.getDataStoreList(self.getRecords()))
        self.assertEqual(expected_list, dictionary_list)

        expected_first_call = call(api_instance=ANY,
                                   dataset_id=self.dataset_id,
                                   starting_record_id='',
                                   batch_size=50000)
        self.assertEqual(expected_first_call, read_api.read_batch_in_sequence.mock_calls[0])
        expected_additional_call = call(api_instance=ANY,
                                        dataset_id=self.dataset_id,
                                        starting_record_id='2.' + self.dataset_id + '.recordId',
                                        batch_size=50000)
        self.assertEqual(expected_additional_call, read_api.read_batch_in_sequence.mock_calls[1])
        self.assertEqual(expected_additional_call, read_api.read_batch_in_sequence.mock_calls[2])

    def test_read_dataset_to_pandas_dataframe(self):
        self.setup()
        read_api = ReadApi()
        read_api.read_batch_in_sequence = Mock()
        # Returning different values for each call to read_batch_in_sequence.
        kdp_conn = KdpConn()
        config = kdp_conn.create_configuration(self.jwt)
        read_api.read_batch_in_sequence.side_effect = [self.getRecordBatch(), self.getRecordBatch(),
                                                       self.getRecordBatchNoMoreRecords()]
        dataframe = read_api.read_dataset_to_pandas_dataframe(config=config,
                                                              dataset_id=self.dataset_id,
                                                              starting_record_id=self.starting_record_id,
                                                              batch_size=self.batch_size)
        expected_list = []
        expected_list.extend(self.getDataStoreList(self.getRecords()))
        expected_list.extend(self.getDataStoreList(self.getRecords()))
        expected_list.extend(self.getDataStoreList(self.getRecords()))
        expected_dataframe = pd.DataFrame(expected_list)

        assert_frame_equal(expected_dataframe, dataframe)

    @staticmethod
    def getRecords():
        # Including data to test that the dataframe gets the correct data
        first_json_record = JsonRecord(record={'a': 3})
        second_json_record = JsonRecord(record={'b': 4})
        return [first_json_record, second_json_record]

    @staticmethod
    def getDataStoreList(json_record_list):
        result_list = []
        for json_record in json_record_list:
            result_list.append(json_record['_data_store'])

        return result_list


    def getRecordBatch(self):
        record_batch = RecordBatch()
        record_batch.records = self.getRecords()
        record_batch.last_record_id = '2.' + self.dataset_id + '.recordId'
        record_batch.more = bool(True)
        return record_batch

    def getRecordBatchNoMoreRecords(self):
        record_batch = RecordBatch()
        record_batch.records = self.getRecords()
        record_batch.last_record_id = '3.' + self.dataset_id + '.recordId'
        record_batch.more = bool(False)
        return record_batch


if __name__ == '__main__':
    unittest.main()



import json
import urllib3
import logging
from urllib3.util import Timeout

class IngestJobApi(object):
    def __init__(self, configuration=None):
        self.configuration = configuration
        host=self.configuration.host.replace('https://', '')

        self.http = urllib3.HTTPSConnectionPool(host, port=443, cert_reqs='CERT_NONE',
            assert_hostname=False, timeout=Timeout(connect=2.0, read=10.0))


    def create_url_ingest_job(self, workspace_id: str, dataset_id: str, url_list) -> str :

        """This method will be used to start a job that ingests files to KDP

            :param str workspace_id: ID of KDP workspace data will be written to
            :param str dataset_id: ID of the KDP dataset where the data will be written
            :param list url_list: List of urls for each file to be ingested

            :returns: Job ID

            :rtype: str
        """

        ingest_request={}
        ingest_request['workspaceId']=workspace_id
        ingest_request['datasetId']=dataset_id
        ingest_request['securityLabeled']=False
        ingest_request['dataSourceParams']={}
        ingest_request['dataSourceParams']['type']='URL'
        ingest_request['dataSourceParams']['connectionInfo']={}
        ingest_request['dataSourceParams']['connectionInfo']['urls']=[]

        for url_str in url_list:
            url={}
            url['url']=url_str
            ingest_request['dataSourceParams']['connectionInfo']['urls'].append(url);

        return self.create_ingest_job(ingest_request)


    def create_ingest_job(self, ingest_request: object) -> str :
        encoded_data = json.dumps(ingest_request).encode('utf-8')

        result: urllib3.response.HTTPResponse = self.http.request(
            'POST',
            self.configuration.host + "/ingest",
            body=encoded_data,
            headers={
                'Content-Type': 'application/json',
                'Authorization': 'Bearer ' + self.configuration.access_token
                }
            )

        if result.status == 202:
            return json.loads(result.data)
        else:
            logging.error('unexpected response code returned. status: %s, reason: %s, message: %s' %
                (result.status, result.reason, result.msg))
            raise Exception('Failed to create ingest job')




from kdp_connector.configuration.configurationUtil import ConfigurationUtil
from kdp_connector.configuration.authenticationUtil import AuthenticationUtil
from kdp_connector.connectors.batch_write import WriteApi
from kdp_connector.connectors.ingest_job_api import IngestJobApi
from kdp_connector.connectors.read import ReadApi
from kdp_connector.connectors.kdp_api import KdpApi
from kdp_connector.connectors.query import QueryApi
from kdp_connector.connectors.index_management import IndexManagementApi
from kdp_connector.connectors.upload import UploadApi


class KdpConn(object):
    """This class contains convenience methods used for interacting with KDP"""

    def __init__(self, path_to_ca_file: str = '', host: str = 'https://api.app.koverse.com',
                 discard_unknown_keys: bool = True):
        self.path_to_ca_file = path_to_ca_file
        self.host = host
        self.discard_unknown_keys = discard_unknown_keys

    def create_configuration(self, jwt: str):
        """This method will be used to create the connection configuration
            :param str jwt: JWT token

            :returns: KDP connection configuration

            :rtype: Configuration
        """
        config = ConfigurationUtil()
        return config.create_configuration(self.host, jwt, self.path_to_ca_file, self.discard_unknown_keys)

    # Auth
    def create_authentication_token(self, email: str, password: str, workspace_id: str,
                                    strategy: str = 'local') -> object:
        """This method will be used to authenticate to KDP

            :param str email: User email
            :param str password: User password
            :param str workspace_id: User workspace
            :param str strategy: Defaults to "local"

            :returns: Authentication token

            :rtype: AuthenticationDetails
        """
        config = self.create_configuration('')

        auth_util = AuthenticationUtil()
        auth_response = auth_util.create_authentication_token(config, email, password, workspace_id, strategy)
        return auth_response


    # Auth, only applicable if jwt is created for auth-proxy
    def create_proxy_authentication_token(self, first_name: str, workspace_id: str, jwt: str = None,
                                    strategy: str = 'proxy') -> object:
        """This method will be used to authenticate to KDP. Only request from auth-proxy with be accepted.

            :param str first_name: User's first name
            :param str workspace_id: User workspace
            :param str strategy: Defaults to "proxy"

            :returns: Authentication token

            :rtype: AuthenticationDetails
        """
        config = self.create_configuration(jwt=jwt)

        auth_util = AuthenticationUtil()
        auth_response = auth_util.create_proxy_authentication_token(config, first_name, workspace_id, strategy)
        return auth_response



    # WRITE
    def batch_write(self, dataframe, dataset_id: str, jwt: str, batch_size: int = 100, is_async: bool = True):
        """This method will be used to write batches of data to KDP

            :param DataFrame dataframe: Data to write to KDP
            :param str dataset_id: ID of the KDP dataset where the data will be written
            :param str jwt: JWT token
            :param int batch_size: Defaults to 100
            :param bool is_async: Defaults to True

            :returns: Set of partitions data was written to

            :rtype: set
        """
        write_api = WriteApi()
        config = self.create_configuration(jwt)
        return write_api.batch_write(config=config, dataset_id=dataset_id, dataframe=dataframe, batch_size=batch_size, is_async=is_async)

    def create_url_ingest_job(self, workspace_id: str, dataset_id: str, url_list, jwt: str) -> str:
        """This method will be used to start a job that ingests files to KDP

            :param str workspace_id: ID of KDP workspace data will be written to
            :param str dataset_id: ID of the KDP dataset where the data will be written
            :param list url_list: List of urls for each file to be ingested
            :param str jwt: JWT token

            :returns: Job ID

            :rtype: str
        """
        config = self.create_configuration(jwt)
        ingest_job_api = IngestJobApi(configuration=config)
        return ingest_job_api.create_url_ingest_job(workspace_id=workspace_id, dataset_id=dataset_id, url_list=url_list)

    # Query
    def post_lucene_query(self, dataset_id: str, jwt: str, expression: str = '', limit: int = 5, offset: int = 0):
        """This method will be used to query data in KDP datasets using the lucene syntax

            :param str dataset_id: ID of the KDP dataset where the data will queried
            :param str jwt: JWT token
            :param str expression: Lucene style query expression ex. name: John

            :returns: Records matching query expression

            :rtype: RecordBatch
        """
        query_api = QueryApi()
        config = self.create_configuration(jwt)
        return query_api.post_lucene_query(config, dataset_id=dataset_id, expression=expression, limit=limit,
                                           offset=offset)

    # READ
    def read_dataset_to_dictionary_list(self, dataset_id: str, jwt: str,
                                        starting_record_id: str = '', batch_size: int = 100000):
        """This method will read records from a dataset to a dictionary list

            :param str dataset_id: ID of the KDP dataset where the data will be read from
            :param str jwt: JWT token
            :param str starting_record_id: First record id to read
            :param int batch_size: Defaults to 100000

            :returns: Dictionary list of records

            :rtype: list
        """
        read_api = ReadApi()
        config = self.create_configuration(jwt)
        return read_api.read_dataset_to_dictionary_list(config, dataset_id, starting_record_id,
                                                        batch_size)

    def read_dataset_to_pandas_dataframe(self, dataset_id: str, jwt: str,
                                         starting_record_id: str = '', batch_size: int = 100000):
        """This method will read KDP dataset records into a pandas dataframe

            :param str dataset_id: ID of the KDP dataset where the data will be read from
            :param str jwt: JWT token
            :param str starting_record_id: First record id to read
            :param int batch_size: Defaults to 100000

            :returns: Pandas dataframe with KDP records

            :rtype: DataFrame
        """
        read_api = ReadApi()
        config = self.create_configuration(jwt)
        return read_api.read_dataset_to_pandas_dataframe(config, dataset_id, starting_record_id,
                                                         batch_size)

    def get_splits(self, dataset_id: str, jwt: str):
        """This method will get a list of splits from for the dataset

            :param str dataset_id: ID of the KDP dataset to get splits from
            :param str jwt: JWT token

            :returns: List of split points

            :rtype: SplitPoints
        """
        read_api = ReadApi()
        config = self.create_configuration(jwt)
        return read_api.get_splits(config=config, dataset_id=dataset_id)

    def read_batch(self, dataset_id: str, starting_record_id: str, ending_record_id: str,
                   exclude_starting_record_id: bool, batch_size: int, jwt: str):
        """This method will read a batch of records from a KDP dataset

            :param str dataset_id: ID of the KDP dataset that will be read from
            :param str starting_record_id: First record id to read
            :param str ending_record_id: Last record id to read
            :param bool exclude_starting_record_id: Whether to exclude starting record id
            :param int batch_size: Size of batch to read
            :param str jwt: JWT token

            :returns: List of records

            :rtype: RecordBatch
        """
        read_api = ReadApi()
        config = self.create_configuration(jwt)
        return read_api.read_batch(config=config, dataset_id=dataset_id, starting_record_id=starting_record_id,
                                   ending_record_id=ending_record_id,
                                   exclude_starting_record_id=exclude_starting_record_id,
                                   batch_size=batch_size)

    # dataset
    def create_dataset(self, name: str, workspace_id: str, jwt: str, description: str = '',
                       auto_create_indexes: bool = True, schema: str = '{}', search_any_field: bool = True,
                       record_count: int = 0):
        """This method will create a new KDP dataset

            :param str name: Name of dataset to create
            :param str workspace_id: Workspace that dataset will be created in
            :param str jwt: JWT token
            :param str description: Description of dataset
            :param bool auto_create_indexes: Whether to automatically index new data
            :param str schema: Schema of dataset
            :param bool search_any_field: Whether to search any field
            :param int record_count: Whether to search any field

            :returns: New dataset

            :rtype: Dataset
        """
        kdp_api = KdpApi()
        config = self.create_configuration(jwt)
        return kdp_api.create_dataset(config, name, workspace_id, description, auto_create_indexes, schema,
                                      search_any_field, record_count)

    def get_dataset(self, dataset_id, jwt):
        """This method will get a dataset by id

            :param str dataset_id: ID of dataset
            :param str jwt: JWT token

            :returns: Dataset

            :rtype: Dataset
        """
        kdp_api = KdpApi()
        config = self.create_configuration(jwt)
        return kdp_api.get_dataset(config, dataset_id)

    def patch_dataset(self, dataset_id, payload, jwt):
        """This method will update fields in a dataset

            :param str dataset_id: ID of dataset
            :param PatchDataset payload: Payload with the fields to update
            :param str jwt: JWT token

            :returns: Dataset

            :rtype: Dataset
        """
        kdp_api = KdpApi()
        config = self.create_configuration(jwt)
        return kdp_api.patch_dataset(config, dataset_id, payload)

    # workspace
    def get_workspace(self, workspace_id, jwt):
        """This method will get a workspace by id

            :param str workspace_id: ID of workspace
            :param str jwt: JWT token

            :returns: Workspace

            :rtype: Workspace
        """
        kdp_api = KdpApi()
        config = self.create_configuration(jwt)
        return kdp_api.get_workspace(config, workspace_id)

    def create_workspace(self, name, jwt):
        """This method will create a new KDP workspace

            :param str name: Name of workspace to create
            :param str jwt: JWT token

            :returns: New workspace

            :rtype: Workspace
        """
        kdp_api = KdpApi()
        config = self.create_configuration(jwt)
        return kdp_api.create_workspace(config, name)

    def delete_workspace(self, workspace_id, jwt):
        """This method will delete a workspace by id

            :param str workspace_id: ID of workspace
            :param str jwt: JWT token

            :returns: Deleted workspace

            :rtype: Workspace
        """
        kdp_api = KdpApi()
        config = self.create_configuration(jwt)
        return kdp_api.delete_workspace(config, workspace_id)

    # indexes
    def get_indexes(self, dataset_id: str, jwt: str, limit: int = 10):
        """This method will get indexes for a dataset

            :param str dataset_id: ID of dataset
            :param str jwt: JWT token
            :param int limit: Limit number of results returned (default 10)

            :returns: Paginator with indexes

            :rtype: IndexPaginator
        """
        kdp_api = KdpApi()
        config = self.create_configuration(jwt)
        return kdp_api.get_indexes(config, dataset_id, limit)

    # modify index
    def modify_indexes(self, dataset_id: str, create: list, remove: list,
                       autoCreateIndexes: bool, searchAnyField: bool, jwt: str) -> object:
        """This method will modify existing indexes on a dataset

            :param str dataset_id: ID of dataset
            :param list create: List of indexes to create
            :param list remove: List of indexes to delete
            :param bool autoCreateIndexes: Whether to automatically create indexes when data is written
            :param bool searchAnyField: Whether to automatically search any field
            :param str jwt: JWT token

            :returns: Job ID

            :rtype: str
       """
        config = self.create_configuration(jwt)
        index_management_api = IndexManagementApi(configuration=config)
        return index_management_api.modify_indexes(dataset_id=dataset_id, create=create, remove=remove,
                                                   autoCreateIndexes=autoCreateIndexes, searchAnyField=searchAnyField)

    # Jobs
    def get_jobs(self, dataset_id: str, jwt: str, **kwargs):
        """This method will get a list of all jobs for a dataset

            :param str dataset_id: ID of dataset
            :param str jwt: JWT token
            :param kwargs:
                See below
            :Keyword Args:
                workspace_id (str): workspaceId. [optional]
                limit (int): Number of results to return. [optional]
                skip (int): Number of results to skip. [optional]
                sort ({str: (bool, date, datetime, dict, float, int, list, str, none_type)}): Property to sort results. [optional]
                filter ({str: (bool, date, datetime, dict, float, int, list, str, none_type)}): Query parameters to filter. [optional]
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (int/float/tuple): timeout setting for this request. If
                    one number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _spec_property_naming (bool): True if the variable names in the input data
                    are serialized names, as specified in the OpenAPI document.
                    False if the variable names in the input data
                    are pythonic names, e.g. snake case (default)
                _content_type (str/None): force body content-type.
                    Default is None and content-type will be predicted by allowed
                    content-types and body.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.

            :returns: A paginator with a list of jobs

            :rtype: JobPaginator
        """
        kdp_api = KdpApi()
        config = self.create_configuration(jwt)
        return kdp_api.get_jobs(config, dataset_id, **kwargs)

    # Upload
    def upload(self, dataset_id: str, file_config: object, jwt: str):
        """This method will upload a file to KDP

            :param str dataset_id: ID of dataset that file will be uploaded to
            :param object file_config: JSON containing filename and path ex. { "filename": "test.csv", "path": "/path/to/file" }
            :param str jwt: JWT token

        """
        config = self.create_configuration(jwt)
        upload_api = UploadApi(configuration=config)
        return upload_api.upload(dataset_id=dataset_id, file_config=file_config)

    # User
    def delete_user(self, user_id: str, jwt: str):
        """This method will delete a user by id

            :param str user_id: ID of user
            :param str jwt: JWT token

            :returns: Deleted user

            :rtype: User
        """
        kdp_api = KdpApi()
        config = self.create_configuration(jwt)
        return kdp_api.delete_user(config, user_id)



from kdp_api.model.authentication import Authentication

"""
This class extends Authentication and override the allowed_values field to allow
authentication request coming from auth-proxy service.
"""
class ProxyAuthentication(Authentication):
    allowed_values = {
        ('strategy',): {
            'PROXY': "proxy",
        },
    }


    def __init__(self, first_name: str, workspace_id: str, strategy: str, *args, **kwargs):
        super().__init__(email="", password="", firstName=first_name, workspaceId=workspace_id, strategy=strategy)



import kdp_api


class ConfigurationUtil(object):

    @staticmethod
    def create_configuration(host: str, jwt: str, path_to_ca_file: str = '', discard_unknown_keys: bool = True):
        """This method will be used to create the connection configuration

            :param str host: KDP host
            :param str jwt: JWT token
            :param str path_to_ca_file: Path to Certificate Authority file
            :param bool discard_unknown_keys: Whether to discard unknown keys, defaults to True

            :returns: KDP connection configuration

            :rtype: Configuration
        """
        # Defining the host is optional and defaults to https://api.app.koverse.com
        # See configurationUtil.py for a list of all supported configuration parameters.
        config = kdp_api.Configuration(
            host=host
        )

        # The client must configure the authentication and authorization parameters
        # in accordance with the API server security policy.
        # Examples for each auth method are provided below, use the example that
        # satisfies your auth use case.

        # provide JWT
        config.access_token = jwt

        ##############################
        ##### SSL configuration  #####
        ##############################
        ## Option 1: disable verifying SSL by not providing a file path to ca file. This will result in warning messages in the log.
        config.verify_ssl = path_to_ca_file != ''

        ## Option 2: specify the location to the CA file (path_to_ca_file). If you are using mkcert, you can find out by running "mkcert -CAROOT" command.
        if config.verify_ssl:
            config.ssl_ca_cert = path_to_ca_file
            config.assert_hostname = False

        ##### end of SSL configuration #####

        # When true will not fail deserializing extra properties that are not defined in schema.
        config.discard_unknown_keys = discard_unknown_keys

        return config



import kdp_api
from kdp_api.api.authentication_api import AuthenticationApi
from kdp_api.api.authentication_api import Authentication
from kdp_connector.configuration.proxy_authentication import ProxyAuthentication

class AuthenticationUtil(object):

    @staticmethod
    def create_authentication_token(config, email: str, password: str, workspace_id: str, strategy: str = 'local'):
        """This method will be used to create a KDP authentication token

            :param Configuration config: KDP configuration
            :param str email: User email address
            :param str password: User password
            :param str workspace_id: User workspace
            :param str strategy: Defaults to "local"

            :returns: Authentication token

            :rtype: AuthenticationDetails
        """

        with kdp_api.ApiClient(config) as api_client:
            api_instance = AuthenticationApi(api_client)

            authentication = Authentication(strategy=strategy, email=email, password=password, workspaceId=workspace_id)
            return api_instance.post_authentication(authentication=authentication)


    @staticmethod
    # only applicable if jwt is created for auth-proxy
    def create_proxy_authentication_token(config, first_name: str, workspace_id: str, strategy: str = 'proxy'):
        """This method will be used to create a KDP authentication token. Only request from auth-proxy with be accepted.

            :param str first_name: User's first name
            :param str workspace_id: User workspace
            :param str strategy: Defaults to "proxy"

            :returns: Authentication token

            :rtype: AuthenticationDetails
        """

        with kdp_api.ApiClient(config) as api_client:
            api_instance = AuthenticationApi(api_client)

            authentication = ProxyAuthentication(strategy=strategy, first_name=first_name, workspace_id=workspace_id)
            return api_instance.post_authentication(authentication=authentication)



import kdp_api as kdp_api_client
from kdp_api.api.datasets_api import DatasetsApi
from kdp_api.api.workspaces_api import WorkspacesApi
from kdp_api.api.indexes_api import IndexesApi
from kdp_api.api.jobs_api import JobsApi
from kdp_api.api.users_api import UsersApi

class KdpApi(object):

    @staticmethod
    def create_dataset(config, name: str, workspace_id: str, description: str = '', auto_create_indexes: bool = True,
                       schema: str = '{}', search_any_field: bool = True, record_count: int = 0):
        """This method will create a new KDP dataset

            :param str name: Name of dataset to create
            :param str workspace_id: Workspace that dataset will be created in
            :param str description: Description of dataset
            :param bool auto_create_indexes: Whether to automatically index new data
            :param str schema: Schema of dataset
            :param bool search_any_field: Whether to search any field
            :param int record_count: Whether to search any field

            :returns: New dataset

            :rtype: Dataset
        """
        with kdp_api_client.ApiClient(config) as api_client:
            request_body = {
                    'name': name,
                    'record_count': record_count,
                    'description': description,
                    'auto_create_indexes': auto_create_indexes,
                    'schema': schema,
                    'search_any_field': search_any_field,
                    'workspace_id': workspace_id
                }

            datasets_api = DatasetsApi(api_client)
            return datasets_api.post_datasets(dataset_create_request=request_body)

    @staticmethod
    def create_workspace(config, name: str, workspace_id: str = None):
        """This method will create a new KDP workspace

            :param str name: Name of workspace to create
            :param str workspace_id: ID of workspace if specified (defaults to name)

            :returns: New workspace

            :rtype: Workspace
        """
        with kdp_api_client.ApiClient(config) as api_client:

            request_body = {
                'name': name,
                'id': workspace_id if workspace_id is not None else name
            }
            workspaces_api = WorkspacesApi(api_client)
            return workspaces_api.post_workspaces(workspace_create_request=request_body)

    @staticmethod
    def delete_workspace(config, workspace_id: str):
        """This method will delete a workspace by id

            :param Configuration config: Connection configuration
            :param str workspace_id: ID of workspace

            :returns: Deleted workspace

            :rtype: Workspace
        """
        with kdp_api_client.ApiClient(config) as api_client:

            workspaces_api = WorkspacesApi(api_client)
            return workspaces_api.delete_workspaces_id(workspace_id)


    @staticmethod
    def get_workspace(config, workspace_id: str):
        """This method will get a workspace by id
            :param Configuration config: Connection configuration
            :param str workspace_id: ID of workspace

            :returns: Workspace

            :rtype: Workspace
        """
        with kdp_api_client.ApiClient(config) as api_client:

            workspaces_api = WorkspacesApi(api_client)
            return workspaces_api.get_workspaces_id(workspace_id)

    @staticmethod
    def get_dataset(config, dataset_id: str):
        """This method will get a dataset by id

            :param Configuration config: Connection configuration
            :param str dataset_id: ID of dataset

            :returns: Dataset

            :rtype: Dataset
        """
        with kdp_api_client.ApiClient(config) as api_client:

            datasets_api = DatasetsApi(api_client)
            return datasets_api.get_datasets_id(dataset_id)


    @staticmethod
    def patch_dataset(config, dataset_id: str, payload):
        """This method will update fields in a dataset

           :param Configuration config: Connection configuration
           :param str dataset_id: ID of dataset
           :param PatchDataset payload: Payload with the fields to update

           :returns: Dataset

           :rtype: Dataset
       """
        with kdp_api_client.ApiClient(config) as api_client:
            datasets_api = DatasetsApi(api_client)
            return datasets_api.patch_datasets_id(id=dataset_id, dataset_patch_request=payload)


    @staticmethod
    def get_indexes(config, dataset_id: str, limit: int = 10):
        """This method will get indexes for a dataset

            :param Configuration config: Connection configuration
            :param str dataset_id: ID of dataset
            :param int limit: Limit number of results returned (default 10)

            :returns: Paginator with indexes

            :rtype: IndexPaginator
        """
        with kdp_api_client.ApiClient(config) as api_client:
            indexes_api = IndexesApi(api_client)
            return indexes_api.get_indexes(dataset_id=dataset_id, limit=limit)

    @staticmethod
    def get_index(config, index_id: str):
        """This method will get an index by ID

            :param Configuration config: Connection configuration
            :param str index_id: ID of index

            :returns: Index object

            :rtype: Index
        """
        with kdp_api_client.ApiClient(config) as api_client:
            indexes_api = IndexesApi(api_client)
            return indexes_api.get_indexes_id(id=index_id)

    @staticmethod
    def get_jobs(config, dataset_id: str, **kwargs):
        """This method will get a list of all jobs for a dataset

            :param Configuration config: Connection configuration
            :param str dataset_id: ID of dataset
            :param kwargs:
                See below
            :Keyword Args:
                workspace_id (str): workspaceId. [optional]
                limit (int): Number of results to return. [optional]
                skip (int): Number of results to skip. [optional]
                sort ({str: (bool, date, datetime, dict, float, int, list, str, none_type)}): Property to sort results. [optional]
                filter ({str: (bool, date, datetime, dict, float, int, list, str, none_type)}): Query parameters to filter. [optional]
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (int/float/tuple): timeout setting for this request. If
                    one number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _spec_property_naming (bool): True if the variable names in the input data
                    are serialized names, as specified in the OpenAPI document.
                    False if the variable names in the input data
                    are pythonic names, e.g. snake case (default)
                _content_type (str/None): force body content-type.
                    Default is None and content-type will be predicted by allowed
                    content-types and body.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.

            :returns: A paginator with a list of jobs

            :rtype: JobPaginator
        """
        with kdp_api_client.ApiClient(config) as api_client:
            jobs_api = JobsApi(api_client)
            return jobs_api.get_jobs(dataset_id=dataset_id, **kwargs)


    @staticmethod
    def delete_user(config, user_id: str):
        """This method will delete a user by id

            :param Configuration config: Connection configuration
            :param str user_id: ID of user

            :returns: Deleted user

            :rtype: User
        """
        with kdp_api_client.ApiClient(config) as api_client:
            user_api = UsersApi(api_client)
            return user_api.delete_users_id(user_id)



from sys import api_version
import kdp_api
from kdp_api.api import query_api

class QueryApi(object):

    def post_lucene_query(self, config, dataset_id: str, expression: str, limit: int = 5, offset: int = 0):
        """This method will be used to query data in KDP datasets using the lucene syntax

            :param Configuration config: Connection configuration
            :param str dataset_id: ID of the KDP dataset where the data will queried
            :param str expression: Lucene style query expression ex. name: John

            :returns: Records matching query expression

            :rtype: RecordBatch
        """
        with kdp_api.ApiClient(config) as api_client:
            api_instance = query_api.QueryApi(api_client)

            query = {}
            query['datasetId'] = dataset_id
            query['expression'] = expression
            query['limit'] = limit
            query['offset'] = offset

            return api_instance.post_lucene_query(lucene_query_request=query)




import pandas as pd
import kdp_api
from kdp_api.api import read_api
from kdp_api.models import SequenceReadRequest
from kdp_api.models import RecordBatch


class ReadApi(object):

    def read_dataset_to_dictionary_list(self, config, dataset_id: str,
                                        starting_record_id: str = '', batch_size: int = 1000):
        """This method will read records from a dataset to a dictionary list

            :param Configuration config: Connection configuration
            :param str dataset_id: ID of the KDP dataset where the data will be read from
            :param str starting_record_id: First record id to read
            :param int batch_size: Defaults to 1000

            :returns: Dictionary list of records

            :rtype: list
        """
        with kdp_api.ApiClient(config) as api_client:
            api_instance = read_api.ReadApi(api_client)
            has_more_records = bool(True)
            dictionary_list = []

            while has_more_records:
                record_batch: RecordBatch = self.read_batch_in_sequence(
                    api_instance=api_instance,
                    dataset_id=dataset_id,
                    starting_record_id=starting_record_id,
                    batch_size=batch_size)
                has_more_records = record_batch.more
                starting_record_id = record_batch.last_record_id
                for json_record in record_batch.records:
                    dictionary_list.append(json_record['_data_store'])

            return dictionary_list

    @staticmethod
    def read_batch_in_sequence(api_instance, dataset_id: str, starting_record_id: str,
                               batch_size: int):
        sequence_read_request = SequenceReadRequest(dataset_id=dataset_id,
                                                    starting_record_id=starting_record_id,
                                                    batch_size=batch_size)
        return api_instance.post_read_in_sequence(read_range_request=sequence_read_request)

    def read_dataset_to_pandas_dataframe(self, config, dataset_id: str, starting_record_id: str = '',
                                         batch_size: int = 100000):
        """This method will read KDP dataset records into a pandas dataframe

            :param Configuration config: Connection configuration
            :param str dataset_id: ID of the KDP dataset where the data will be read from
            :param str starting_record_id: First record id to read
            :param int batch_size: Defaults to 100000

            :returns: Pandas dataframe with KDP records

            :rtype: DataFrame
        """
        dictionary_list = self.read_dataset_to_dictionary_list(config, dataset_id, starting_record_id,
                                                               batch_size)
        return pd.DataFrame(dictionary_list)

    @staticmethod
    def get_splits(config, dataset_id: str):
        """This method will get a list of splits from for the dataset

            :param Configuration config: Connection configuration
            :param str dataset_id: ID of the KDP dataset to get splits from

            :returns: List of split points

            :rtype: SplitPoints
        """
        with kdp_api.ApiClient(config) as api_client:
            api_instance = read_api.ReadApi(api_client)

            return api_instance.get_splits_id(dataset_id=dataset_id)

    @staticmethod
    def read_batch(config, dataset_id: str, starting_record_id: str, ending_record_id: str,
                   exclude_starting_record_id: bool, batch_size: int = 10):
        """This method will read a batch of records from a KDP dataset

            :param Configuration config: Connection configuration
            :param str dataset_id: ID of the KDP dataset that will be read from
            :param str starting_record_id: First record id to read
            :param str ending_record_id: Last record id to read
            :param bool exclude_starting_record_id: Whether to exclude starting record id
            :param int batch_size: Size of batch to read (default is 10)

            :returns: List of records

            :rtype: RecordBatch
        """
        with kdp_api.ApiClient(config) as api_client:
            api_instance = read_api.ReadApi(api_client)

            request = {}
            request['datasetId'] = dataset_id
            request['excludeStartingRecordId'] = exclude_starting_record_id
            request['startingRecordId'] = starting_record_id
            request['endingRecordId'] = ending_record_id
            request['batchSize'] = batch_size

            return api_instance.post_read(request)
